//@author: antokhio
//@help: template for standard shaders
//@tags: template
//@credits: 

//StructuredBuffer<float4> color <bool color=true;>;

struct Brush
{
	float4x4 Transform;
	int TexID;
};

StructuredBuffer<Brush> sbBrush;

Texture2DArray tex0;
SamplerState s0:IMMUTABLE {Filter=MIN_MAG_MIP_LINEAR;AddressU=CLAMP;AddressV=CLAMP;};

float4x4 tW : WORLD;
float4x4 tVP : VIEWPROJECTION;

float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
float4 cAmb <bool color=true; String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
float4x4 tTex <bool uvspace=true; string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

uint InstanceStartIndex;

struct VS_IN
{
	float4 pos : POSITION;
	float4 uv : TEXCOORD0;
	uint ii : SV_InstanceID;
};

struct VS_OUT
{
    float4 pos: SV_POSITION;
    float4 uv: TEXCOORD0;
	uint id : TEXCOORD1; 
};

VS_OUT VS(VS_IN input)
{
    VS_OUT output = (VS_OUT)0;
    
	float4x4 world = sbBrush[input.ii + InstanceStartIndex].Transform;
	
	output.pos  = mul(input.pos,mul(world,tVP));
    output.uv = mul(input.uv, tTex);
	
	output.id = sbBrush[input.ii + InstanceStartIndex].TexID;
	
    return output;
}

float4 PS(VS_OUT input): SV_Target
{
    float4 col = tex0.Sample(s0,float3(input.uv.xy,input.id)) * cAmb;
	col.a *= Alpha;
    return col;
}

technique10 Template
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}




